package cn.com.ty.lift.business.maintenance.service;

import cn.com.ty.lift.business.maintenance.dao.entity.MaintenancePlan;
import cn.com.ty.lift.business.maintenance.dao.entity.model.UpdateMaintenancePlanReq;
import cn.com.ty.lift.business.maintenance.dao.mapper.MaintenancePlanMapper;
import cn.com.ty.lift.business.project.dao.entity.Project;
import cn.com.ty.lift.business.project.service.ProjectService;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author bieao
 * @date 2019/12/14
 * @description 维保计划业务层
 */
@Service
public class MaintenancePlanService extends ServiceImpl<MaintenancePlanMapper, MaintenancePlan> {

    @Resource
    private ProjectService projectService;

    @Resource
    private MaintenancePlanMapper planMapper;

    /**
     * @param planList 电梯列表
     * @return 是否成功
     * @description 批量生成维保计划
     * @date 2019/12/16 2:14 PM
     */
    public boolean insertBatch(List<MaintenancePlan> planList) {
        List<MaintenancePlan> resultList = generatePlan(planList);
        return saveBatch(resultList, resultList.size());
    }

    /**
     * @param beginTime 项目开始时间
     * @param endTime   项目结束时间
     * @param interval  保养间隔
     * @return 实际期数
     * @description 计算当前电梯所在项目开始时间到结束时间需要保养的期数
     * @date 2019/12/16 2:11 PM
     */
    public int calculatePeriods(Date beginTime, Date endTime, int interval) {
        long result = DateUtil.betweenDay(beginTime, endTime, false);
        Double num = Math.floor(Double.valueOf(String.valueOf((result / interval))));
        return num.intValue();
    }

    /**
     * @param plans 需要保养的电梯
     * @return List<MaintenancePlan> 维保计划列表
     * @description 生成维保计划
     * @date 2019/12/16 1:21 PM
     */
    public List<MaintenancePlan> generatePlan(List<MaintenancePlan> plans) {
        //返回维保计划列表
        List<MaintenancePlan> planList = new ArrayList<>();
        //保养类型 1.半月，2.季度，3.半年，4.全年
        int[] maintenanceType = {1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 3, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 4};
        int times = 0;
        for (MaintenancePlan plan : plans) {
            //获取从第几期开始
            int periods = plan.getPeriods();
            //获取保养间隔
            int interval = plan.getInterval();
            Project project = projectService.detail(plan.getProjectId());
            if (project != null) {
                Date beginTime = project.getStartDate();
                Date endTime = project.getEndDate();
                //计算当前电梯所在项目开始时间到结束时间需要保养的期数
                times = calculatePeriods(beginTime, endTime, interval);
            }
            for (int i = 0; i < times; i++) {
                MaintenancePlan entry = new MaintenancePlan();
                //设置维保计划电梯ID，维保公司ID，项目ID，维保工ID，维保类型
                entry.setLiftId(plan.getLiftId());
                entry.setMtCompanyId(plan.getMtCompanyId());
                entry.setProjectId(plan.getProjectId());
                entry.setWorkerId(plan.getWorkerId());
                entry.setType(maintenanceType[periods - 1]);
                if (i == 0) {
                    entry.setPlanDate(plan.getPlanDate());
                } else {
                    //设置维护保养时间
                    MaintenancePlan maintenancePlan = planList.get(i - 1);
                    Date planDate = DateUtil.offsetDay(maintenancePlan.getPlanDate(), interval);
                    entry.setPlanDate(planDate);
                }
                System.out.printf("i：%s,期数：%s,类型：%s,计划时间：%s,%s", (i + 1), periods, entry.getType(), entry.getPlanDate(), "\n");
                periods++;
                if (periods == 25) {
                    periods = 1;
                }
                planList.add(entry);
            }
        }
        return planList;
    }

    /**
     * @param request 修改维保计划请求
     * @return 是否修改成功
     * @description 修改维保计划
     * @date 2019/12/16 5:25 PM
     */
    public boolean modifyPlan(UpdateMaintenancePlanReq request) {
        List<MaintenancePlan> resultList = new ArrayList<>();
        //获取当前保养时间
        Date currentTime = request.getCurrentTime();
        //获取修改的保养时间
        Date updateTime = request.getUpdateTime();
        //获取偏移量
        int result = DateUtil.compare(currentTime, updateTime);
        //获取时间间隔
        Long interval = DateUtil.betweenDay(currentTime, updateTime, false);
        int offset = interval.intValue();
        //获取当前时间后的保养时间
        List<MaintenancePlan> idList = planMapper.findIdList(currentTime);
        for (MaintenancePlan entry : idList) {
            Date planDate;
            if (result > 0) {
                planDate = DateUtil.offsetDay(entry.getPlanDate(), -offset);
            } else {
                planDate = DateUtil.offsetDay(entry.getPlanDate(), offset);
            }
            entry.setPlanDate(planDate);
            resultList.add(entry);
        }
       return updateBatchById(resultList, resultList.size());
    }
}
